/*
 * playlist2html.C - convert an m3u playlist into an html playlist
 * Copyright (c) 2002 Justin Spadea (jzs@mail.rit.edu)
 *
 * If you don't have an m3u already, open up XMMS, load up the playlist editor, and save it.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 */

#include "conf.h"
#include <fstream.h>
#include <stdlib.h>
#include <iomanip.h>
#include <string>
#include <stdio.h>

using namespace std;


void processFile(char* infile, char* outfile){

  ifstream streamfile( infile );

  if ( streamfile.is_open() ) {
    
    int count(0);
    string lineIn, body, clipping, final2;
    string final = 
      "<HTML>\n"
      "<HEAD>\n"
      "<TITLE>MP3 Playlist</TITLE>\n"
      "<style type=\"text/css\">\n"
      "<!--\n"
      "A {color:red; text-decoration:none;}\n"
      "A:hover {text-decoration:underline;}\n"
      ".title {font-size:80px; font-weight:bold; color:#00A9FB; position:relative; z-index:1000;}\n"
      ".title2 {font-size:80px; font-weight:bold; color:#0080C0; position:absolute; top:20px; left:12px;}\n"
      ".bottom {font-size:20px; color:#0080C0;}\n"
      ".maintable {border-top:1px solid white; border-bottom:1px solid white;}\n"
      "//-->\n"
      "</style>\n"
      "</HEAD>\n"
      "<BODY BGCOLOR=\"#000000\" TEXT=\"#ffffff\">\n"
      "<div class=\"title\">MP3 Playlist</div>\n"
      "<font color=\"#95E879\" size=\"1\">\n"
      "Total tracks: ";


    // process m3u - generate body of html
    while( !streamfile.eof() ) {
      getline( streamfile, lineIn );
      if(lineIn != "#EXTM3U"){       //this is first line of m3u files - skip it
	clipping = lineIn.substr(0,8);
	if(clipping == "#EXTINF:"){
	  count++;
	  int pos = lineIn.find(",");  //song title starts after the comma
	  clipping = lineIn.substr(++pos);
	  body += "<br>" + clipping + "\n";
	}
      }
    }
    streamfile.close();

    // we can continue now that we have a count of files
    final2 = "</font><br>\n"
      "<table width=\"90%\" class=\"maintable\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\" align=\"center\">\n"
      "<tr><td>\n"
      "<font size=\"2\">\n";
    
    // add the mp3 files
    final2 += body;

    // and now the final tags
    final2 += "</font>\n"
      "</td></tr>\n"
      "</table>\n"
      "</BODY>\n"
      "</HTML>";

    
    // save file
    char *outputFileName( outfile );                       // name of file to be saved
    ofstream ostreamfile( outputFileName, ios::trunc  );
    if ( ostreamfile.is_open() ) {
      ostreamfile << final << count << endl << final2;
      ostreamfile.close();
    } else {
      printf("File %s was unopenable for some reason.\n", outfile);
    }
  } else {
    printf("File %s does not exist. Exiting.\n", infile);
  }
}


void printVersion(char* name){
  printf("%s version: %s\n", name, VERSION);
  exit(0);
}


void printUsage(char* name){
  printf("Usage: %s [OPTIONS] -f file\n", name);
}


void printHelp(char* name){
  printUsage(name);
  printf("\nOPTIONS:\n-f, --file <string>                       playlist input file (*.m3u)\n"
	 "-o, --ofile <string>                      html output file (not required)\n"
	 "-v, --version                             display version number and exit\n"
	 "-h, --help                                display this help message\n"
	 "\n");
  printVersion(name);
  exit(0);
}


void checkFile(char* infile, char* outfile){
  char bakup[100];
  strcpy(bakup, infile);               // use backup copy so original filename isn't changed by strtok()
  char* temp = strtok(bakup, ".");     // before the .
  temp = strtok(NULL, "."); 	       // after the .
  
  if(!strcmp(temp, "m3u")){
    // good, its an m3u
    processFile(infile, outfile);
  } else {
    printf("Error: %s does not end with .m3u.\n"
	   "If this is an xmms/winamp playlist file, rename it to have the proper extension.\n", infile);
  }
   
}


int main( int argc, char **argv ) {

  char* infile("");
  char* outfile("");

  // no args given
  if(argc == 1){
    printUsage(argv[0]);
    printf("Type \"%s --help\" for options\n", argv[0]);
  } else {
    
    // process cmd line args
    for(int i(1); i<argc; i++){
      if(!strcmp(argv[i], "-f") || !strcmp(argv[i], "-file") || !strcmp(argv[i], "--file")){
	infile = argv[++i];
      }
      else if(!strcmp(argv[i], "-o") || !strcmp(argv[i], "-ofile") || !strcmp(argv[i], "--ofile")){
	outfile = argv[++i];
      }
      else if(!strcmp(argv[i], "-v") || !strcmp(argv[i], "-version") || !strcmp(argv[i], "--version")){
	printVersion(argv[0]);
      }
      else if(!strcmp(argv[i], "-h") || !strcmp(argv[i], "-help") || !strcmp(argv[i], "--help")){
	printHelp(argv[0]);
      }
      else {
	printf("Invalid option %s\n"
	       "Type \"%s --help\" for options\n", argv[i], argv[0]);
      }
    }
    
    // use default output if none given
    if(outfile == "") outfile = OUTNAME;
    
    // process file or exit if none
    if(infile != ""){
      checkFile(infile, outfile);
    }
  }
  exit(0);
}
